@@ -45,7 +45,7 @@ enum connman_device_type {
 	CONNMAN_DEVICE_TYPE_NOZOMI    = 16,
 	CONNMAN_DEVICE_TYPE_HUAWEI    = 17,
 	CONNMAN_DEVICE_TYPE_NOVATEL   = 18,
-	CONNMAN_DEVICE_TYPE_VENDOR    = 42,
+	CONNMAN_DEVICE_TYPE_VENDOR    = 10000,
 };
 
 enum connman_device_mode {
@@ -70,8 +70,7 @@ extern struct connman_device *connman_device_create(const char *node,
 extern struct connman_device *connman_device_ref(struct connman_device *device);
 extern void connman_device_unref(struct connman_device *device);
 
-extern void connman_device_set_path(struct connman_device *device,
-							const char *path);
+extern const char *connman_device_get_name(struct connman_device *device);
 extern const char *connman_device_get_path(struct connman_device *device);
 extern void connman_device_set_index(struct connman_device *device,
 								int index);
@@ -84,6 +83,7 @@ extern void connman_device_set_policy(struct connman_device *device,
 					enum connman_device_policy policy);
 extern void connman_device_set_mode(struct connman_device *device,
 						enum connman_device_mode mode);
+extern enum connman_device_mode connman_device_get_mode(struct connman_device *device);
 
 extern int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered);
@@ -91,6 +91,13 @@ extern int connman_device_set_carrier(struct connman_device *device,
 						connman_bool_t carrier);
 extern int connman_device_set_scanning(struct connman_device *device,
 						connman_bool_t scanning);
+extern int connman_device_set_disconnected(struct connman_device *device,
+						connman_bool_t disconnected);
+
+extern int connman_device_set_string(struct connman_device *device,
+					const char *key, const char *value);
+extern const char *connman_device_get_string(struct connman_device *device,
+							const char *key);
 
 extern int connman_device_add_network(struct connman_device *device,
 					struct connman_network *network);
